﻿
Public Class Form1

    ' Declare the Sleep function from Kernel32.dll
    Private Declare Sub Sleep Lib "Kernel32" (ByVal dwMilliseconds As Integer)

    ' Array of CheckBoxes for AI channels
    Dim checkboxAi(0 To 23) As CheckBox
    ' Array of TextBoxes for AI values
    Dim textboxAiValue(0 To 23) As TextBox
    ' Array of CheckBoxes for DI channels
    Dim checkboxDi(0 To 3) As CheckBox
    ' Array of CheckBoxes for DO channels
    Dim checkboxDo(0 To 3) As CheckBox

    ' Device index
    Dim DevIndex As Integer
    ' Flag to indicate if the program is running
    Dim bRun As Boolean


    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        ' Temporary variable
        Dim temp As Integer
        ' Loop variable
        Dim i As Integer

        ' Find available USB5 DAQ devices
        temp = FindUSB5DAQ()
        If (temp > 0) Then

            ' Add device indices to the combo box
            For i = 0 To temp - 1
                comboboxDeviceIndex.Items.Add(i)
            Next

            ' Select the first device index
            comboboxDeviceIndex.SelectedIndex = 0
            ' Enable the start button
            btnStart.Enabled = True
        Else
            ' Disable the start button if no devices are found
            btnStart.Enabled = False
        End If

        ' Create and configure CheckBoxes and TextBoxes for AI channels
        For i = 0 To 15
            checkboxAi(i) = New CheckBox
            checkboxAi(i).Width = 60
            checkboxAi(i).Text = "Ai " & i.ToString
            checkboxAi(i).Left = 20 + 75 * (i \ 8)
            checkboxAi(i).Top = 20 + (i Mod 8) * 20
            GroupBox1.Controls.Add(checkboxAi(i))

            textboxAiValue(i) = New TextBox
            textboxAiValue(i).Width = 50
            textboxAiValue(i).Left = 20 + 55 * (i Mod 8)
            textboxAiValue(i).Top = 20 + (i \ 8) * 60
            GroupBox2.Controls.Add(textboxAiValue(i))
        Next

        ' Check the first 8 AI channels
        For i = 0 To 7
            checkboxAi(i).Checked = True
        Next

        ' Disable the stop button
        btnStop.Enabled = False
        ' Set the running flag to false
        bRun = False

    End Sub

    Private Sub btnStart_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnStart.Click

        ' Loop variables
        Dim i As Integer
        Dim j As Integer
        ' Temporary variable
        Dim temp As Integer
        ' Array to store the device serial number
        Dim SN(8) As Byte

        ' Get the selected device index
        DevIndex = comboboxDeviceIndex.SelectedIndex

        ' Open the device with the specified index
        temp = USB5OpenDevice(DevIndex)

        ' Get the device serial number
        temp = USB5GetDeviceSN(DevIndex, SN(0))
        textboxSN.Text = "SN: " + System.Text.Encoding.ASCII.GetString(SN)

        '-----------------------------------------
        ' AI related settings

        ' Configure 24 AI channels
        For i = 0 To 15
            ' Select whether to enable the channel
            temp = SetUSB5AiChanSel(DevIndex, i, CByte(checkboxAi(i).CheckState))
            ' Set the range of the channel
            temp = SetUSB5AiRange(DevIndex, i, 10.24)

            ' Clear all AI displays
            textboxAiValue(i).Text = ""
        Next

        ' Set the acquisition mode to continuous acquisition mode
        temp = SetUSB5AiSampleMode(DevIndex, 0)

        ' Set the AI input sampling period to 100000 ns, i.e., 100us, and the sampling rate to 10kHz
        temp = SetUSB5AiSampleRate(DevIndex, 100000)

        ' Set the AI trigger source to software trigger
        temp = SetUSB5AiTrigSource(DevIndex, 0)

        ' Set the AI sampling clock to the AI internal sampling clock
        temp = SetUSB5AiConvSource(DevIndex, 0)

        ' Set the AI pre-trigger points to 0
        temp = SetUSB5AiPreTrigPoints(DevIndex, 0)

        ' Clear the AI FIFO once
        temp = SetUSB5ClrAiFifo(DevIndex)

        ' Start a software trigger
        temp = SetUSB5AiSoftTrig(DevIndex)


        ' Start Timer1
        Timer1.Interval = 5
        Timer1.Enabled = True
        ' Disable the start button
        btnStart.Enabled = False
        ' Enable the stop button
        btnStop.Enabled = True
        ' Set the running flag to true
        bRun = True

    End Sub

    Private Sub Timer1_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer1.Tick

        ' Loop variables
        Dim i As Integer
        Dim j As Integer
        ' Temporary variable
        Dim temp As Integer
        ' Array to store AI data
        Static Dim ai(4800000) As Single
        ' Array to store DI data
        Dim di(100) As Byte
        ' Variable to store the average value
        Dim average As Single

        '------------------------------------------------
        ' Analog Input related functions

        ' Read 1000 numbers from each channel in the AI FIFO and store them in the ai array
        ' The data read out each time will be a continuous set of data obtained from continuous acquisition
        temp = USB5GetAi(DevIndex, 1000, ai(0), 1000)

        ' Display the number of data points remaining in the AI FIFO
        ' In long-term continuous acquisition applications, pay attention to the change of the temp value
        ' If the temp value is gradually increasing, it means that the data processing speed is not fast enough, resulting in data积压 in the AI FIFO
        textboxFifoNum.Text = temp

        temp = 0
        ' Calculate the total number of selected channels in this acquisition
        For i = 0 To 15
            temp = temp + CByte(checkboxAi(i).CheckState)
        Next

        ' Calculate the average value of the measured voltage for each channel and display it
        For j = 0 To temp - 1
            average = 0
            For i = 0 To 999
                average = average + ai(j * 1000 + i)
            Next
            average = average / 1000
            textboxAiValue(j).Text = Format(average, "0.0000")
        Next

    End Sub

    Private Sub btnStop_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnStop.Click

        ' Close the device
        CloseDevice()

    End Sub

    Private Sub Form1_FormClosing(ByVal sender As System.Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles MyBase.FormClosing

        ' If the program is running, close the device
        If bRun = True Then
            CloseDevice()
        End If

    End Sub

    Private Sub CloseDevice()

        ' Temporary variable
        Dim temp As Integer

        ' Disable Timer1
        Timer1.Enabled = False
        ' Sleep for 100 milliseconds
        Sleep(100)

        ' Clear the hardware trigger flag
        temp = SetUSB5ClrAiTrigger(DevIndex)
        ' Clear the AI FIFO
        temp = SetUSB5ClrAiFifo(DevIndex)

        ' Close the device
        temp = USB5CloseDevice(DevIndex)

        ' Enable the start button
        btnStart.Enabled = True
        ' Disable the stop button
        btnStop.Enabled = False
        ' Set the running flag to false
        bRun = False

    End Sub
End Class
