﻿
Public Class Form1

    ' Declare the Sleep function from the Kernel32 library to pause the program for a specified number of milliseconds.
    Private Declare Sub Sleep Lib "Kernel32" (ByVal dwMilliseconds As Integer)

    ' Array of CheckBox controls for analog input channels (AI), from index 0 to 23.
    Dim checkboxAi(0 To 23) As CheckBox
    ' Array of TextBox controls to display the values of analog input channels (AI), from index 0 to 23.
    Dim textboxAiValue(0 To 23) As TextBox
    ' Array of CheckBox controls for digital input channels (DI), from index 0 to 3.
    Dim checkboxDi(0 To 3) As CheckBox
    ' Array of CheckBox controls for digital output channels (DO), from index 0 to 3.
    Dim checkboxDo(0 To 3) As CheckBox

    ' Index of the selected device.
    Dim DevIndex As Integer
    ' Boolean flag indicating whether the device is running or not.
    Dim bRun As Boolean

    ' Event handler for the form load event.
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        ' Temporary variable used for storing intermediate results.
        Dim temp As Integer
        ' Loop variable for iterating through device indices.
        Dim i As Integer

        ' Find all available USB5 DAQ devices and get the number of devices.
        temp = FindUSB5DAQ()
        If (temp > 0) Then
            ' Add the indices of all available devices to the comboboxDeviceIndex control.
            For i = 0 To temp - 1
                comboboxDeviceIndex.Items.Add(i)
            Next
            ' Select the first device in the combobox by default.
            comboboxDeviceIndex.SelectedIndex = 0
            ' Enable the start button to allow starting operations on the device.
            btnStart.Enabled = True
        Else
            ' Disable the start button if no devices are found.
            btnStart.Enabled = False
        End If
    End Sub

    ' Event handler for the click event of the start button.
    Private Sub btnStart_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnStart.Click

        ' Loop variables for iterating through arrays and performing operations.
        Dim i As Integer
        Dim j As Integer
        ' Temporary variable used for storing intermediate results of function calls.
        Dim temp As Integer
        ' Array to store the serial number of the device.
        Dim SN(8) As Byte
        ' Array to store the waveform data for analog output.
        Dim WaveData(100) As Single

        ' Get the index of the selected device from the combobox.
        DevIndex = comboboxDeviceIndex.SelectedIndex

        ' Open the device with the specified index.
        temp = USB5OpenDevice(DevIndex)

        ' Get the serial number of the opened device and display it in the textbox.
        temp = USB5GetDeviceSN(DevIndex, SN(0))
        textboxSN.Text = "SN: " + System.Text.Encoding.ASCII.GetString(SN)

        '-----------------------------------------
        ' Settings related to the analog output (Ao) waveform.

        ' Clear the global software trigger flag.
        temp = SetUSB5ClrGlobalSoftTrig(DevIndex)

        ' Clear the trigger flags of the 4 analog output (Ao) channels respectively.
        For i = 0 To 3
            temp = SetUSB5ClrAoTrigger(DevIndex, i)
        Next

        ' Clear the hardware FIFO spaces of the 4 analog output (Ao) channels respectively.
        For i = 0 To 3
            temp = SetUSB5ClrAoFifo(DevIndex, i)
        Next

        ' Set the output modes of the 4 analog output (Ao) channels to waveform output with hardware FIFO buffering respectively.
        For i = 0 To 3
            temp = SetUSB5AoSampleMode(DevIndex, i, 1)
        Next

        ' Set the sampling periods of the 4 analog output (Ao) channels to 10000 ns (i.e., 10us), with a sampling rate of 100kHz respectively.
        For i = 0 To 3
            temp = SetUSB5AoSampleRate(DevIndex, i, 1000)
        Next

        ' Set the trigger sources of the 4 analog output (Ao) channels to global software trigger respectively.
        For i = 0 To 3
            temp = SetUSB5AoTrigSource(DevIndex, i, 255)
        Next

        ' Set the sampling clocks of the 4 analog output (Ao) channels to the internal Ao sampling clock respectively.
        For i = 0 To 3
            temp = SetUSB5AoConvSource(DevIndex, i, 0)
        Next

        ' Set the number of cycles for the output waveforms of the 4 analog output (Ao) channels to 0, indicating infinite loop output respectively.
        ' If the number of cycles is set to 1, the waveform data will stop outputting after one complete cycle.
        For i = 0 To 3
            temp = SetUSB5AoCycle(DevIndex, i, 0)
        Next

        ' Generate a waveform consisting of 100 random numbers between 0 and 1V and store it in the WaveData array.
        For i = 0 To 99
            WaveData(i) = Rnd()
        Next

        ' Download the waveform data of 100 points (WaveData) to the hardware Ao FIFOs of the 4 channels respectively.
        For i = 0 To 3
            temp = SetUSB5AoDataFifo(DevIndex, i, WaveData(0), 100)
        Next

        ' Enable the channels that need to perform synchronous sampling output.
        ' Each bit of &HF represents a channel. &HF means all 4 channels are enabled for synchronous sampling output.
        temp = SetUSB5AoSync(DevIndex, &HF)

        ' Trigger all channels with global software trigger.
        temp = SetUSB5GlobalSoftTrig(DevIndex)

        ' Close the device after all operations are completed.
        temp = USB5CloseDevice(DevIndex)
    End Sub
End Class
