﻿Public Class Form1

    ' Declare the Sleep function from the Kernel32 library to pause the program for a specified number of milliseconds.
    Private Declare Sub Sleep Lib "Kernel32" (ByVal dwMilliseconds As Integer)

    ' Array of CheckBox controls for analog input channels (AI), from index 0 to 23.
    Dim checkboxAi(0 To 23) As CheckBox
    ' Array of TextBox controls to display the values of analog input channels (AI), from index 0 to 23.
    Dim textboxAiValue(0 To 23) As TextBox
    ' Array of CheckBox controls for digital input channels (DI), from index 0 to 3.
    Dim checkboxDi(0 To 3) As CheckBox
    ' Array of CheckBox controls for digital output channels (DO), from index 0 to 3.
    Dim checkboxDo(0 To 3) As CheckBox

    ' Index of the selected device.
    Dim DevIndex As Integer
    ' Boolean flag indicating whether the device is running or not.
    Dim bRun As Boolean

    ' Array to store the waveform data for analog output, with a length of 1000 points.
    Dim WaveData(1000) As Single
    ' Unsigned long variable to record the number of waveform data update rounds.
    Dim Rounds As ULong
    ' Thread object used to download waveform data continuously.
    Dim threadDownload As Threading.Thread


    ' Event handler for the form load event.
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        ' Temporary variable used for storing intermediate results.
        Dim temp As Integer
        ' Loop variable for iterating through device indices.
        Dim i As Integer

        ' Find all available USB5 DAQ devices and get the number of devices.
        temp = FindUSB5DAQ()
        If (temp > 0) Then
            ' Add the indices of all available devices to the comboboxDeviceIndex control.
            For i = 0 To temp - 1
                comboboxDeviceIndex.Items.Add(i)
            Next
            ' Select the first device in the combobox by default.
            comboboxDeviceIndex.SelectedIndex = 0
            ' Enable the start button to allow starting operations on the device.
            btnStart.Enabled = True
        Else
            ' Disable the start button if no devices are found.
            btnStart.Enabled = False
        End If

        ' Disable the stop button initially.
        btnStop.Enabled = False

    End Sub

    ' Event handler for the click event of the start button.
    Private Sub btnStart_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnStart.Click

        ' Loop variables for iterating through arrays and performing operations.
        Dim i As Integer
        Dim j As Integer
        ' Temporary variable used for storing intermediate results of function calls.
        Dim temp As Integer
        ' Array to store the serial number of the device.
        Dim SN(8) As Byte

        ' Generate a waveform consisting of 1000 random numbers between 0 and 1V and store it in the WaveData array.
        For i = 0 To 999
            WaveData(i) = Rnd()
        Next

        ' Get the index of the selected device from the combobox.
        DevIndex = comboboxDeviceIndex.SelectedIndex

        ' Open the device with the specified index.
        temp = USB5OpenDevice(DevIndex)

        ' Get the serial number of the opened device and display it in the textbox.
        temp = USB5GetDeviceSN(DevIndex, SN(0))
        textboxSN.Text = System.Text.Encoding.ASCII.GetString(SN)

        '-----------------------------------------
        ' Settings related to the analog output (Ao) waveform.

        ' Clear the trigger flag of analog output channel 0 (Ao_0).
        temp = SetUSB5ClrAoTrigger(DevIndex, 0)

        ' Clear the hardware FIFO space of analog output channel 0 (Ao_0).
        temp = SetUSB5ClrAoFifo(DevIndex, 0)

        ' Set the output mode of analog output channel 0 (Ao_0) to the infinite-length non-cyclic waveform output mode with computer buffering.
        temp = SetUSB5AoSampleMode(DevIndex, 0, 0)

        ' Set the sampling period of analog output channel 0 (Ao_0) to 10000 ns (i.e., 10us), with a sampling rate of 100kHz.
        temp = SetUSB5AoSampleRate(DevIndex, 0, 1000)

        ' Set the trigger source of analog output channel 0 (Ao_0) to software trigger.
        temp = SetUSB5AoTrigSource(DevIndex, 0, 0)

        ' Set the sampling clock of analog output channel 0 (Ao_0) to the internal Ao sampling clock.
        temp = SetUSB5AoConvSource(DevIndex, 0, 0)

        ' Download the waveform data of 1000 points (WaveData) to the hardware FIFO.
        temp = SetUSB5AoDataFifo(DevIndex, 0, WaveData(0), 1000)

        ' Disable the start button after starting the operation.
        btnStart.Enabled = False
        ' Enable the stop button to allow stopping the operation.
        btnStop.Enabled = True
        ' Initialize the number of rounds to 0.
        Rounds = 0
        ' Set the running flag to true to indicate the device is running.
        bRun = True

        ' Set the interval of Timer1 to 100 milliseconds and enable it.
        Timer1.Interval = 100
        Timer1.Enabled = True

        ' Trigger the analog output of channel 0 (Ao_0) using software.
        temp = SetUSB5AoSoftTrig(DevIndex, 0)

        ' Create a new thread to continuously generate new data and download it to the hardware FIFO of the acquisition card.
        threadDownload = New Threading.Thread(AddressOf DownloadWaveData)
        threadDownload.Start()

    End Sub

    ' Method executed by the thread to download waveform data continuously.
    Private Sub DownloadWaveData()

        ' Temporary variable used for storing intermediate results of function calls.
        Dim temp As Integer
        ' Loop variable for iterating through the waveform data array.
        Dim i As Integer

        ' Continuously execute the loop while the device is running.
        Do While bRun = True

            ' Update the waveform data.
            For i = 0 To 999
                WaveData(i) = Rnd()
            Next

            ' Download the updated waveform data of 1000 points (WaveData) to the hardware FIFO.
            temp = SetUSB5AoDataFifo(DevIndex, 0, WaveData(0), 1000)

            ' Increment the number of rounds.
            Rounds = Rounds + 1

            ' Since the data of 1000 points can be output for 10ms, delay for 5ms before generating new data and downloading it to ensure the continuous output of the waveform without interruption.
            Sleep(5)
        Loop

    End Sub

    ' Event handler for the click event of the stop button.
    Private Sub btnStop_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnStop.Click

        ' Close the device.
        CloseDevice()
        ' Enable the start button again to allow restarting the operation.
        btnStart.Enabled = True
        ' Disable the stop button.
        btnStop.Enabled = False

    End Sub

    ' Method to close the device and release resources.
    Private Sub CloseDevice()

        ' Temporary variable used for storing intermediate results of function calls.
        Dim temp As Integer

        ' Check if the device is running.
        If bRun = True Then
            ' Set the running flag to false to stop the data generation and download thread.
            bRun = False
            ' Wait for the thread to finish executing.
            threadDownload.Join()

            ' Clear the trigger flag of analog output channel 0 (Ao_0).
            temp = SetUSB5ClrAoTrigger(DevIndex, 0)

            ' Clear the hardware FIFO of analog output channel 0 (Ao_0).
            temp = SetUSB5ClrAoFifo(DevIndex, 0)

            ' Close the device.
            temp = USB5CloseDevice(DevIndex)
        End If

    End Sub

    ' Event handler for the form closing event.
    Private Sub Form1_FormClosing(ByVal sender As System.Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles MyBase.FormClosing

        ' Close the device when the form is closing.
        CloseDevice()

    End Sub

    ' Event handler for the Tick event of Timer1.
    Private Sub Timer1_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer1.Tick

        ' Display the number of rounds in the textbox.
        textboxRounds.Text = Rounds

    End Sub
End Class