﻿Public Class Form1

    ' Declare the Sleep function from the Kernel32 library to pause the program for a specified number of milliseconds.
    Private Declare Sub Sleep Lib "Kernel32" (ByVal dwMilliseconds As Integer)

    ' Array of CheckBox controls for analog input channels (AI), from index 0 to 23.
    Dim checkboxAi(0 To 23) As CheckBox
    ' Array of TextBox controls to display the values of analog input channels (AI), from index 0 to 23.
    Dim textboxAiValue(0 To 23) As TextBox
    ' Array of CheckBox controls for digital input channels (DI), from index 0 to 3.
    Dim checkboxDi(0 To 3) As CheckBox
    ' Array of CheckBox controls for digital output channels (DO), from index 0 to 3.
    Dim checkboxDo(0 To 3) As CheckBox

    ' Index of the selected device.
    Dim DevIndex As Integer
    ' Boolean flag indicating whether the device is running or not.
    Dim bRun As Boolean


    ' Event handler for the form load event.
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        ' Temporary variable used for storing intermediate results.
        Dim temp As Integer
        ' Loop variable for iterating through device indices.
        Dim i As Integer

        ' Find all available USB5 DAQ devices and get the number of devices.
        temp = FindUSB5DAQ()
        If (temp > 0) Then
            ' Add the indices of all available devices to the comboboxDeviceIndex control.
            For i = 0 To temp - 1
                comboboxDeviceIndex.Items.Add(i)
            Next
            ' Select the first device in the combobox by default.
            comboboxDeviceIndex.SelectedIndex = 0
            ' Enable the start button to allow starting operations on the device.
            btnStart.Enabled = True
        Else
            ' Disable the start button if no devices are found.
            btnStart.Enabled = False
        End If

        ' Create and configure CheckBox controls for digital input channels (DI).
        For i = 0 To 3
            checkboxDi(i) = New CheckBox
            checkboxDi(i).Text = "Di " + i.ToString
            checkboxDi(i).Top = 40
            checkboxDi(i).Left = 100 + i * 150
            GroupBox1.Controls.Add(checkboxDi(i))
        Next

        ' Disable the stop button initially.
        btnStop.Enabled = False
        ' Set the running flag to false.
        bRun = False

    End Sub

    ' Event handler for the click event of the start button.
    Private Sub btnStart_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnStart.Click

        ' Loop variables for iterating through arrays and performing operations.
        Dim i As Integer
        Dim j As Integer
        ' Temporary variable used for storing intermediate results of function calls.
        Dim temp As Integer
        ' Array to store the serial number of the device.
        Dim SN(8) As Byte

        ' Get the index of the selected device from the combobox.
        DevIndex = comboboxDeviceIndex.SelectedIndex

        ' Open the device with the specified index.
        temp = USB5OpenDevice(DevIndex)

        ' Get the serial number of the opened device and display it in the textbox.
        temp = USB5GetDeviceSN(DevIndex, SN(0))
        textboxSN.Text = "SN: " + System.Text.Encoding.ASCII.GetString(SN)

        '-----------------------------------------
        ' Settings related to digital input (DI).

        ' Set the acquisition mode to continuous acquisition mode.
        temp = SetUSB5DiSampleMode(DevIndex, 0)

        ' Set the sampling period of DI input to 10000 ns (i.e., 10us), with a sampling rate of 100kHz.
        temp = SetUSB5DiSampleRate(DevIndex, 10000)

        ' Set the trigger source of DI to software trigger.
        temp = SetUSB5DiTrigSource(DevIndex, 0)

        ' Set the sampling clock of DI to the internal DI sampling clock.
        temp = SetUSB5DiConvSource(DevIndex, 0)

        ' Set the number of pre-trigger points of DI to 0.
        temp = SetUSB5DiPreTrigPoints(DevIndex, 0)

        ' Clear the DI FIFO once.
        temp = SetUSB5ClrDiFifo(DevIndex)

        ' Start a software trigger.
        temp = SetUSB5DiSoftTrig(DevIndex)

        ' Start Timer1.
        Timer1.Interval = 5
        Timer1.Enabled = True
        ' Disable the start button after starting the operation.
        btnStart.Enabled = False
        ' Enable the stop button to allow stopping the operation.
        btnStop.Enabled = True
        ' Set the running flag to true to indicate the device is running.
        bRun = True

    End Sub

    ' Event handler for the Tick event of Timer1.
    Private Sub Timer1_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer1.Tick

        ' Loop variables for iterating through arrays and performing operations.
        Dim i As Integer
        Dim j As Integer
        ' Temporary variable used for storing intermediate results of function calls.
        Dim temp As Integer
        ' Array to store the digital input data.
        Dim di(5000) As Byte
        ' Variable to store the average value (not used in this code for DI).
        Dim average As Single

        '------------------------------------------------
        ' Functions related to digital input.

        ' Read 1000 numbers from each channel in the DI FIFO and store them in the di array.
        ' The data read out each time will be a continuous set of data obtained from continuous acquisition.
        temp = USB5GetDi(DevIndex, 5000, di(0), 1000)

        ' Display the number of data points remaining in the DI FIFO.
        ' In long-term continuous acquisition applications, pay attention to the change of the temp value.
        ' If the temp value is gradually increasing, it means that the data processing speed is not fast enough, resulting in data backlog in the DI FIFO.
        textboxFifoNum.Text = temp

        ' Display the latest state of the continuous DI data.
        ' The lower two bits of di correspond to two DI channels.
        For i = 0 To 3
            checkboxDi(i).CheckState = di(4999) And &H1
            di(4999) = di(4999) \ 2
        Next

    End Sub

    ' Event handler for the click event of the stop button.
    Private Sub btnStop_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnStop.Click

        ' Close the device.
        CloseDevice()

    End Sub

    ' Event handler for the form closing event.
    Private Sub Form1_FormClosing(ByVal sender As System.Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles MyBase.FormClosing

        ' If the device is running, close it when the form is closing.
        If bRun = True Then
            CloseDevice()
        End If

    End Sub

    ' Method to close the device and release resources.
    Private Sub CloseDevice()

        ' Temporary variable used for storing intermediate results of function calls.
        Dim temp As Integer

        ' Disable Timer1.
        Timer1.Enabled = False
        ' Pause the program for 100 milliseconds.
        Sleep(100)

        ' Clear the hardware trigger flag.
        temp = SetUSB5ClrDiTrigger(DevIndex)
        ' Clear the DI FIFO.
        temp = SetUSB5ClrDiFifo(DevIndex)

        ' Close the device.
        temp = USB5CloseDevice(DevIndex)

        ' Enable the start button again to allow restarting the operation.
        btnStart.Enabled = True
        ' Disable the stop button.
        btnStop.Enabled = False
        ' Set the running flag to false.
        bRun = False

    End Sub
End Class