﻿Public Class Form1

    ' Declare the Sleep function from the Kernel32 library to pause the program for a specified number of milliseconds.
    Private Declare Sub Sleep Lib "Kernel32" (ByVal dwMilliseconds As Integer)

    ' Array of CheckBox controls for analog input channels (AI), from index 0 to 23.
    Dim checkboxAi(0 To 23) As CheckBox
    ' Array of TextBox controls to display the values of analog input channels (AI), from index 0 to 23.
    Dim textboxAiValue(0 To 23) As TextBox
    ' Array of CheckBox controls for digital input channels (DI), from index 0 to 3.
    Dim checkboxDi(0 To 3) As CheckBox
    ' Array of CheckBox controls for digital output channels (DO), from index 0 to 3.
    Dim checkboxDo(0 To 3) As CheckBox

    ' Index of the selected device.
    Dim DevIndex As Integer
    ' Boolean flag indicating whether the device is running or not.
    Dim bRun As Boolean


    ' Event handler for the form load event.
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        ' Temporary variable used for storing intermediate results.
        Dim temp As Integer
        ' Loop variable for iterating through device indices.
        Dim i As Integer

        ' Find all available USB5 DAQ devices and get the number of devices.
        temp = FindUSB5DAQ()
        If (temp > 0) Then
            ' Add the indices of all available devices to the comboboxDeviceIndex control.
            For i = 0 To temp - 1
                comboboxDeviceIndex.Items.Add(i)
            Next
            ' Select the first device in the combobox by default.
            comboboxDeviceIndex.SelectedIndex = 0
            ' Enable the start button to allow starting operations on the device.
            btnStart.Enabled = True
        Else
            ' Disable the start button if no devices are found.
            btnStart.Enabled = False
        End If
    End Sub

    ' Event handler for the click event of the start button.
    Private Sub btnStart_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnStart.Click

        ' Loop variables for iterating through arrays and performing operations.
        Dim i As Integer
        Dim j As Integer
        ' Temporary variable used for storing intermediate results of function calls.
        Dim temp As Integer
        ' Array to store the serial number of the device.
        Dim SN(8) As Byte
        ' Array to store the digital output waveform data.
        Dim WaveData(100) As UInteger

        ' Get the index of the selected device from the combobox.
        DevIndex = comboboxDeviceIndex.SelectedIndex

        ' Open the device with the specified index.
        temp = USB5OpenDevice(DevIndex)

        ' Get the serial number of the opened device and display it in the textbox.
        temp = USB5GetDeviceSN(DevIndex, SN(0))
        textboxSN.Text = "SN: " + System.Text.Encoding.ASCII.GetString(SN)

        '-----------------------------------------
        ' Settings related to digital output (DO) waveform.

        ' Select the channels for waveform output.
        ' Each bit of &HF corresponds to a channel. &HF means 4 channels are selected.
        temp = SetUSB5DoWaveCtrl(DevIndex, &HF)

        ' Clear the trigger flag of DO.
        temp = SetUSB5ClrDoTrigger(DevIndex)

        ' Clear the hardware FIFO space of DO.
        temp = SetUSB5ClrDoFifo(DevIndex)

        ' Set the output mode of DO to waveform output with hardware FIFO buffering.
        temp = SetUSB5DoSampleMode(DevIndex, 1)

        ' Set the sampling period of DO input to 10000 ns (i.e., 10us), with a sampling rate of 100kHz.
        temp = SetUSB5DoSampleRate(DevIndex, 1000)

        ' Set the trigger source of DO to software trigger.
        temp = SetUSB5DoTrigSource(DevIndex, 0)

        ' Set the sampling clock of DO to the internal Ao sampling clock.
        temp = SetUSB5DoConvSource(DevIndex, 0)

        ' Set the number of cycles for the DO output waveform to 0, indicating infinite loop output.
        ' If the number of cycles is set to 1, the waveform data will stop outputting after one complete cycle.
        temp = SetUSB5DoCycle(DevIndex, 0)

        ' Generate a waveform consisting of 100 random numbers and store it in the WaveData array.
        ' The lower 4 bits of each WaveData element correspond to 4 DO channels.
        For i = 0 To 99
            WaveData(i) = Rnd() * 15
        Next

        ' Download the waveform data of 100 points (WaveData) to the hardware FIFO.
        temp = SetUSB5DoDataFifo(DevIndex, WaveData(0), 100)

        ' Trigger the digital output using software.
        temp = SetUSB5DoSoftTrig(DevIndex)

        ' Close the device.
        temp = USB5CloseDevice(DevIndex)
    End Sub
End Class