﻿Public Class Form1

    ' Declare the Sleep function from the Kernel32 library to pause the program for a specified number of milliseconds.
    Private Declare Sub Sleep Lib "Kernel32" (ByVal dwMilliseconds As Integer)

    ' Array of CheckBox controls for analog input channels (AI), from index 0 to 23.
    Dim checkboxAi(0 To 23) As CheckBox
    ' Array of TextBox controls to display the values of analog input channels (AI), from index 0 to 23.
    Dim textboxAiValue(0 To 23) As TextBox
    ' Array of CheckBox controls for digital input channels (DI), from index 0 to 3.
    Dim checkboxDi(0 To 3) As CheckBox
    ' Array of CheckBox controls for digital output channels (DO), from index 0 to 3.
    Dim checkboxDo(0 To 3) As CheckBox

    ' Index of the selected device.
    Dim DevIndex As Integer
    ' Boolean flag indicating whether the device is running or not.
    Dim bRun As Boolean
    ' Array to store the device model information.
    Dim Model(4) As Byte


    ' Event handler for the form load event.
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        ' Temporary variable used for storing intermediate results.
        Dim temp As Integer
        ' Loop variable for iterating through device indices.
        Dim i As Integer

        ' Find all available USB5 DAQ devices and get the number of devices.
        temp = FindUSB5DAQ()
        If (temp > 0) Then
            ' Add the indices of all available devices to the comboboxDeviceIndex control.
            For i = 0 To temp - 1
                comboboxDeviceIndex.Items.Add(i)
            Next
            ' Select the first device in the combobox by default.
            comboboxDeviceIndex.SelectedIndex = 0
            ' Enable the start button to allow starting operations on the device.
            btnStart.Enabled = True
        Else
            ' Disable the start button if no devices are found.
            btnStart.Enabled = False
        End If

        ' Create and configure CheckBox and TextBox controls for analog input channels (AI).
        For i = 0 To 15
            checkboxAi(i) = New CheckBox
            checkboxAi(i).Width = 60
            checkboxAi(i).Text = "Ai " & i.ToString
            checkboxAi(i).Left = 20 + 75 * (i \ 8)
            checkboxAi(i).Top = 20 + (i Mod 8) * 20
            GroupBox1.Controls.Add(checkboxAi(i))

            textboxAiValue(i) = New TextBox
            textboxAiValue(i).Width = 50
            textboxAiValue(i).Left = 20 + 55 * (i Mod 8)
            textboxAiValue(i).Top = 20 + (i \ 8) * 60
            GroupBox2.Controls.Add(textboxAiValue(i))
        Next

        ' Check the first 8 AI channels by default.
        For i = 0 To 7
            checkboxAi(i).Checked = True
        Next

        ' Initialize the textboxes for analog output channels (AO) to 0.
        textboxAO_0.Text = 0
        textboxAO_1.Text = 0
        textboxAO_2.Text = 0
        textboxAO_3.Text = 0

        ' Create and configure CheckBox controls for digital input channels (DI).
        For i = 0 To 3
            checkboxDi(i) = New CheckBox
            checkboxDi(i).Text = "Di " & i.ToString
            checkboxDi(i).Left = 54
            checkboxDi(i).Top = 20 + 22 * i
            GroupBox4.Controls.Add(checkboxDi(i))
        Next

        ' Create and configure CheckBox controls for digital output channels (DO).
        For i = 0 To 3
            checkboxDo(i) = New CheckBox
            checkboxDo(i).Text = "Do " & i.ToString
            checkboxDo(i).Left = 54
            checkboxDo(i).Top = 20 + 22 * i
            GroupBox5.Controls.Add(checkboxDo(i))
        Next

        ' Disable the stop button initially.
        btnStop.Enabled = False
        ' Set the running flag to false.
        bRun = False
    End Sub

    ' Event handler for the click event of the start button.
    Private Sub btnStart_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnStart.Click

        ' Loop variables for iterating through arrays and performing operations.
        Dim i As Integer
        Dim j As Integer
        ' Temporary variable used for storing intermediate results of function calls.
        Dim temp As Integer
        ' Array to store the serial number of the device.
        Dim SN(8) As Byte

        ' Get the index of the selected device from the combobox.
        DevIndex = comboboxDeviceIndex.SelectedIndex

        ' Open the device with the specified index.
        temp = USB5OpenDevice(DevIndex)

        ' Get the serial number of the opened device and display it in the textbox.
        temp = USB5GetDeviceSN(DevIndex, SN(0))
        textboxSN.Text = "SN: " + System.Text.Encoding.ASCII.GetString(SN)

        ' Get the device model information.
        temp = USB5GetDeviceModel(DevIndex, Model(0))
        textboxModel.Text = "Model: " + System.Text.Encoding.ASCII.GetString(Model)

        If Model(1) = &H31 Or Model(1) = &H32 Then
            ' If the model is 51xx or 52xx, the device has AI functionality.

            '-----------------------------------------
            ' Settings related to analog input (AI).

            ' Configure 16 AI channels.
            For i = 0 To 15
                ' Select whether to enable the channel.
                temp = SetUSB5AiChanSel(DevIndex, i, CByte(checkboxAi(i).CheckState))
                ' Set the range of the channel.
                temp = SetUSB5AiRange(DevIndex, i, 10)

                ' Clear all AI displays.
                textboxAiValue(i).Text = ""
            Next

            ' Set the acquisition mode to OneShot mode.
            temp = SetUSB5AiSampleMode(DevIndex, 1)

            ' Set the number of points to be acquired per channel in OneShot mode to 200.
            temp = SetUSB5AiOneShotPoints(DevIndex, 200)

            ' Set the sampling period of AI input to 10000 ns (i.e., 10us), with a sampling rate of 100kHz.
            temp = SetUSB5AiSampleRate(DevIndex, 10000)

            ' Set the trigger source of AI to software trigger.
            temp = SetUSB5AiTrigSource(DevIndex, 0)

            ' Set the sampling clock of AI to the internal AI sampling clock.
            temp = SetUSB5AiConvSource(DevIndex, 0)

            ' Set the number of pre - trigger points of AI to 0.
            temp = SetUSB5AiPreTrigPoints(DevIndex, 0)

            ' Clear the AI FIFO once.
            temp = SetUSB5ClrAiFifo(DevIndex)
        End If

        If Model(1) = &H31 Or Model(1) = &H32 Then
            ' If the model is 51xx or 52xx, the device has DI functionality.

            '----------------------------------------------
            ' Settings related to digital input (DI).

            ' Set the acquisition mode to OneShot mode.
            temp = SetUSB5DiSampleMode(DevIndex, 1)

            ' Set the number of points to be acquired per channel in OneShot mode to 1.
            temp = SetUSB5DiOneShotPoints(DevIndex, 1)

            ' Set the sampling period of DI input to 10000 ns (i.e., 10us), with a sampling rate of 100kHz.
            temp = SetUSB5DiSampleRate(DevIndex, 10000)

            ' Set the trigger source of DI to software trigger.
            temp = SetUSB5DiTrigSource(DevIndex, 0)

            ' Set the sampling clock of DI to the internal DI sampling clock.
            temp = SetUSB5DiConvSource(DevIndex, 0)

            ' Set the number of pre - trigger points of DI to 0.
            temp = SetUSB5DiPreTrigPoints(DevIndex, 0)

            ' Clear the DI FIFO once.
            temp = SetUSB5ClrDiFifo(DevIndex)
        End If

        ' Start Timer1.
        Timer1.Enabled = True
        ' Disable the start button after starting the operation.
        btnStart.Enabled = False
        ' Enable the stop button to allow stopping the operation.
        btnStop.Enabled = True
        ' Set the running flag to true to indicate the device is running.
        bRun = True
    End Sub

    ' Event handler for the Tick event of Timer1.
    Private Sub Timer1_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer1.Tick

        ' Loop variables for iterating through arrays and performing operations.
        Dim i As Integer
        Dim j As Integer
        ' Temporary variable used for storing intermediate results of function calls.
        Dim temp As Integer
        ' Array to store the analog input data.
        Static Dim ai(4800000) As Single
        ' Array to store the digital input data.
        Dim di(100) As Byte
        ' Variable to store the average value of AI data.
        Dim average As Single

        If Model(1) = &H31 Or Model(1) = &H32 Then
            ' If the model is 51xx or 52xx, the device has AI functionality.

            '------------------------------------------------
            ' Functions related to analog input.

            ' Start a software trigger.
            temp = SetUSB5AiSoftTrig(DevIndex)
            ' Pause the program for 10 milliseconds.
            Sleep(10)
            ' Read 200 numbers from each channel and store them in the ai array.
            temp = USB5GetAi(DevIndex, 200, ai(0), 1000)

            temp = 0
            ' Calculate the total number of selected channels in this acquisition.
            For i = 0 To 15
                temp = temp + CByte(checkboxAi(i).CheckState)
            Next

            ' Calculate the average value of the measured voltage for each channel and display it.
            For j = 0 To temp - 1
                average = 0
                For i = 0 To 199
                    average = average + ai(j * 200 + i)
                Next
                average = average / 200
                textboxAiValue(j).Text = Format(average, "0.0000")
            Next

            ' Clear the hardware trigger flag.
            temp = SetUSB5ClrAiTrigger(DevIndex)
            ' Clear the AI FIFO.
            temp = SetUSB5ClrAiFifo(DevIndex)
        End If

        If Model(1) = &H31 Or Model(1) = &H35 Then
            ' If the model is 51xx, the device has AO functionality.

            '---------------------------------------------
            ' Functions related to analog output.

            If Val(textboxAO_0.Text) < -10 Or Val(textboxAO_0.Text) > 10 Then
                textboxAO_0.Text = 0
            End If
            ' Set the voltage of AO 0.
            temp = SetUSB5AoImmediately(DevIndex, 0, CSng(Val(textboxAO_0.Text)))

            If Val(textboxAO_1.Text) < -10 Or Val(textboxAO_1.Text) > 10 Then
                textboxAO_1.Text = 0
            End If
            ' Set the voltage of AO 1.
            temp = SetUSB5AoImmediately(DevIndex, 1, CSng(Val(textboxAO_1.Text)))

            If Val(textboxAO_2.Text) < -10 Or Val(textboxAO_2.Text) > 10 Then
                textboxAO_2.Text = 0
            End If
            ' Set the voltage of AO 2.
            temp = SetUSB5AoImmediately(DevIndex, 2, CSng(Val(textboxAO_2.Text)))

            If Val(textboxAO_3.Text) < -10 Or Val(textboxAO_3.Text) > 10 Then
                textboxAO_3.Text = 0
            End If
            ' Set the voltage of AO 3.
            temp = SetUSB5AoImmediately(DevIndex, 3, CSng(Val(textboxAO_3.Text)))
        End If

        If Model(1) = &H31 Or Model(1) = &H32 Then
            ' If the model is 51xx or 52xx, the device has DI and DO functionality.

            '----------------------------------------------------
            ' Functions related to digital input.

            ' Start a software trigger.
            temp = SetUSB5DiSoftTrig(DevIndex)

            ' Read the DI status and store it in the di array.
            temp = USB5GetDi(DevIndex, 1, di(0), 100)

            ' Display the DI status.
            ' The lower two bits of di correspond to two DI channels.
            For i = 0 To 3
                checkboxDi(i).CheckState = di(0) And &H1
                di(0) = di(0) \ 2
            Next

            ' Clear the hardware trigger flag.
            temp = SetUSB5ClrDiTrigger(DevIndex)

            ' Clear the DI FIFO.
            temp = SetUSB5ClrDiFifo(DevIndex)

            '----------------------------------------------------
            ' Functions related to digital output.

            For i = 0 To 3
                ' Set the DO output status.
                temp = SetUSB5DoImmediately(DevIndex, i, checkboxDo(i).CheckState)
            Next
        End If
    End Sub

    ' Event handler for the click event of the stop button.
    Private Sub btnStop_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnStop.Click

        ' Close the device.
        CloseDevice()
    End Sub

    ' Event handler for the form closing event.
    Private Sub Form1_FormClosing(ByVal sender As System.Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles MyBase.FormClosing

        ' If the device is running, close it when the form is closing.
        If bRun = True Then
            CloseDevice()
        End If
    End Sub

    ' Method to close the device and release resources.
    Private Sub CloseDevice()

        ' Temporary variable used for storing intermediate results of function calls.
        Dim temp As Integer

        ' Disable Timer1.
        Timer1.Enabled = False
        ' Pause the program for 200 milliseconds.
        Sleep(200)

        ' Close the device.
        temp = USB5CloseDevice(DevIndex)

        ' Enable the start button again to allow restarting the operation.
        btnStart.Enabled = True
        ' Disable the stop button.
        btnStop.Enabled = False
        ' Set the running flag to false.
        bRun = False
    End Sub
End Class